@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from openstack import exceptions as sdk_exc
 from oslo_config import cfg
 from oslo_log import log
 
@@ -32,10 +33,6 @@ class NovaClient(base.DriverBase):
     def flavor_find(self, name_or_id, ignore_missing=False):
         return self.conn.compute.find_flavor(name_or_id, ignore_missing)
 
-    @sdk.translate_exception
-    def image_find(self, name_or_id, ignore_missing=False):
-        return self.conn.compute.find_image(name_or_id, ignore_missing)
-
     @sdk.translate_exception
     def keypair_create(self, **attrs):
         return self.conn.compute.create_keypair(**attrs)
@@ -154,11 +151,13 @@ class NovaClient(base.DriverBase):
 
     @sdk.translate_exception
     def wait_for_server(self, server, status=consts.VS_ACTIVE,
-                        failures=[consts.VS_ERROR],
+                        failures=None,
                         interval=2, timeout=None):
         """Wait for server creation complete"""
+        if failures is None:
+            failures = [consts.VS_ERROR]
         if timeout is None:
-            timeout = cfg.CONF.default_action_timeout
+            timeout = cfg.CONF.default_nova_timeout
 
         server_obj = self.conn.compute.find_server(server, False)
         self.conn.compute.wait_for_server(server_obj, status=status,
@@ -171,7 +170,7 @@ class NovaClient(base.DriverBase):
     def wait_for_server_delete(self, server, timeout=None):
         """Wait for server deleting complete"""
         if timeout is None:
-            timeout = cfg.CONF.default_action_timeout
+            timeout = cfg.CONF.default_nova_timeout
 
         server_obj = self.conn.compute.find_server(server, True)
         if server_obj:
@@ -197,17 +196,26 @@ class NovaClient(base.DriverBase):
         res = self.conn.compute.get_server_metadata(server)
         return res.metadata
 
+    def _ignore_forbidden_call(self, func, *args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except sdk_exc.HttpException as exc:
+            if exc.status_code != 403:
+                raise
+
     @sdk.translate_exception
     def server_metadata_update(self, server, metadata):
         # Clean all existing metadata first
         res = self.conn.compute.get_server_metadata(server)
         if res.metadata:
-            self.conn.compute.delete_server_metadata(
-                server, list(res.metadata.keys()))
-
-        # Then reset metadata to given value if it is not {}
+            for key in res.metadata:
+                self._ignore_forbidden_call(
+                    self.conn.compute.delete_server_metadata, server, [key])
         if metadata:
-            return self.conn.compute.set_server_metadata(server, **metadata)
+            for key, value in metadata.items():
+                self._ignore_forbidden_call(
+                    self.conn.compute.set_server_metadata,
+                    server, **{key: value})
 
     @sdk.translate_exception
     def server_metadata_delete(self, server, keys):
@@ -257,6 +265,41 @@ class NovaClient(base.DriverBase):
     def hypervisor_get(self, hypervisor):
         return self.conn.compute.get_hypervisor(hypervisor)
 
+    @sdk.translate_exception
+    def hypervisor_find(self, name_or_id, ignore_missing=False):
+        # try finding hypervisor by id
+        try:
+            return self.conn.compute.get_hypervisor(name_or_id)
+        except sdk_exc.HttpException:
+            # ignore http exception and instead get list and check by name
+            pass
+
+        # if the hypervisor could not be found using id, search list using name
+        results = self.conn.compute.hypervisors(
+            hypervisor_hostname_pattern=name_or_id)
+
+        result = None
+        for maybe_result in results:
+            name_value = maybe_result.name
+
+            if name_value == name_or_id:
+                # Only allow one resource to be found. If we already
+                # found a match, raise an exception to show it.
+                if result is None:
+                    result = maybe_result
+                else:
+                    msg = "More than one hypervisor exists with the name '%s'."
+                    msg = (msg % name_or_id)
+                    raise sdk_exc.DuplicateResource(msg)
+
+        if result is not None:
+            return result
+
+        if ignore_missing:
+            return None
+        raise sdk_exc.ResourceNotFound(
+            "No hypervisor found for %s" % (name_or_id))
+
     @sdk.translate_exception
     def service_list(self):
         return self.conn.compute.services()
