@@ -62,11 +62,11 @@ class Message(base.Receiver):
             # is not provided in configuration file
             base = self._get_base_url()
             if not base:
-                msg = _('Receiver notification host is not specified in '
-                        'configuration file and Senlin service endpoint can '
-                        'not be found, using local hostname (%(host)s) for '
-                        'subscriber url.') % {'host': host}
-                LOG.warning(msg)
+                LOG.warning('Receiver notification host is not specified in '
+                            'configuration file and Senlin service '
+                            'endpoint can not be found, using local'
+                            ' hostname (%(host)s) for subscriber url.',
+                            {'host': host})
                 host = socket.gethostname()
 
         if not base:
@@ -97,13 +97,15 @@ class Message(base.Receiver):
                                                      self.project,
                                                      roles)
         except exc.InternalError as ex:
-            msg = _('Can not build trust between user %(user)s and zaqar '
-                    'service user %(zaqar)s for receiver %(receiver)s.'
-                    ) % {'user': self.user, 'zaqar': zaqar_trustee_user_id,
-                         'receiver': self.id}
-            LOG.error(msg)
+            LOG.error('Can not build trust between user %(user)s and zaqar '
+                      'service user %(zaqar)s for receiver %(receiver)s.',
+                      {
+                          'user': self.user,
+                          'zaqar': zaqar_trustee_user_id,
+                          'receiver': self.id
+                      })
             raise exc.EResourceCreation(type='trust',
-                                        message=ex.message)
+                                        message=str(ex))
         return trust.id
 
     def _create_queue(self):
@@ -116,7 +118,8 @@ class Message(base.Receiver):
         try:
             self.zaqar().queue_create(**kwargs)
         except exc.InternalError as ex:
-            raise exc.EResourceCreation(type='queue', message=ex.message)
+            raise exc.EResourceCreation(type='queue',
+                                        message=str(ex))
 
         return queue_name
 
@@ -139,7 +142,7 @@ class Message(base.Receiver):
                                                             **kwargs)
         except exc.InternalError as ex:
             raise exc.EResourceCreation(type='subscription',
-                                        message=ex.message)
+                                        message=str(ex))
         return subscription
 
     def _find_cluster(self, context, identity):
@@ -185,25 +188,25 @@ class Message(base.Receiver):
             raise exc.InternalError(message=msg)
 
         # Permission check
-        if not context.is_admin and context.user != cluster_obj.user:
+        if not context.is_admin and self.user != cluster_obj.user:
             msg = _('%(user)s is not allowed to trigger actions on '
-                    'cluster %(cid)s.') % {'user': context.user,
+                    'cluster %(cid)s.') % {'user': self.user,
                                            'cid': cluster}
             raise exc.InternalError(message=msg)
 
-        # Action name check
-        if action not in consts.ACTION_NAMES:
-            msg = _("Illegal action '%s' specified.") % action
-            raise exc.InternalError(message=msg)
+        # Use receiver owner context to build action
+        context.user_id = self.user
+        context.project_id = self.project
+        context.domain_id = self.domain
 
-        if action.lower().split('_')[0] != 'cluster':
-            msg = _("Action '%s' is not applicable to clusters."
-                    ) % action
+        # Action name check
+        if action not in consts.CLUSTER_ACTION_NAMES:
+            msg = _("Illegal cluster action '%s' specified.") % action
             raise exc.InternalError(message=msg)
 
         kwargs = {
             'name': 'receiver_%s_%s' % (self.id[:8], message['id'][:8]),
-            'cause': action_mod.CAUSE_RPC,
+            'cause': consts.CAUSE_RPC,
             'status': action_mod.Action.READY,
             'inputs': params
         }
@@ -233,22 +236,25 @@ class Message(base.Receiver):
         except exc.InternalError as ex:
             raise exc.EResourceDeletion(type='subscription',
                                         id='subscription',
-                                        message=ex.message)
+                                        message=str(ex))
         # Delete zaqar queue
         try:
             self.zaqar().queue_delete(queue_name)
         except exc.InternalError as ex:
             raise exc.EResourceDeletion(type='queue',
                                         id='queue_name',
-                                        message=ex.message)
+                                        message=str(ex))
 
     def notify(self, context, params=None):
+        queue_name = self.channel['queue_name']
         # Claim message(s) from queue
+        # TODO(Yanyanhu) carefully handling claim ttl to avoid
+        # potential race condition.
         try:
-            claim = self.zaqar().claim_create(self.channel['queue_name'])
+            claim = self.zaqar().claim_create(queue_name)
             messages = claim.messages
         except exc.InternalError as ex:
-            LOG.error(_('Failed in claiming message: %s') % ex.message)
+            LOG.error('Failed in claiming message: %s', ex)
             return
 
         # Build actions
@@ -257,14 +263,19 @@ class Message(base.Receiver):
             for message in messages:
                 try:
                     action_id = self._build_action(context, message)
+                    actions.append(action_id)
                 except exc.InternalError as ex:
-                    LOG.error(_('Failed in building action: %s'
-                                ) % ex.message)
-                    continue
-                actions.append(action_id)
-            msg = _('Actions %(actions)s were successfully built.'
-                    ) % {'actions': actions}
-            LOG.info(msg)
+                    LOG.error('Failed in building action: %s', ex)
+                try:
+                    self.zaqar().message_delete(queue_name, message['id'],
+                                                claim.id)
+                except exc.InternalError as ex:
+                    LOG.error('Failed in deleting message %(id)s: %(reason)s',
+                              {'id': message['id'], 'reason': ex})
+
+            self.zaqar().claim_delete(queue_name, claim.id)
+            LOG.info('Actions %(actions)s were successfully built.',
+                     {'actions': actions})
 
             dispatcher.start_action()
 
