@@ -16,7 +16,6 @@ from oslo_utils import strutils
 from oslo_utils import uuidutils
 from oslo_versionedobjects import fields
 import re
-import six
 
 from senlin.common import consts
 from senlin.common.i18n import _
@@ -109,7 +108,7 @@ class UUID(fields.FieldType):
 
 class Json(fields.FieldType):
     def coerce(self, obj, attr, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             try:
                 return jsonutils.loads(value)
             except ValueError:
@@ -124,7 +123,7 @@ class Json(fields.FieldType):
         return jsonutils.dumps(value)
 
     def stringify(self, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             try:
                 return jsonutils.loads(value)
             except ValueError:
@@ -198,7 +197,7 @@ class Capacity(fields.Integer):
 
     def __init__(self, minimum=0, maximum=None):
         super(Capacity, self).__init__()
-        CONF.import_opt("max_nodes_per_cluster", "senlin.common.config")
+        CONF.import_opt("max_nodes_per_cluster", "senlin.conf")
 
         if minimum > CONF.max_nodes_per_cluster:
             err = _("The value of 'minimum' cannot be greater than the global "
@@ -319,7 +318,7 @@ class BaseEnum(fields.FieldType):
             raise ValueError(_("No list of valid values provided for enum."))
 
         for value in valid_values:
-            if not isinstance(value, six.string_types):
+            if not isinstance(value, str):
                 raise ValueError(_("Enum field only support string values."))
 
         self._valid_values = list(valid_values)
@@ -327,7 +326,7 @@ class BaseEnum(fields.FieldType):
         super(BaseEnum, self).__init__()
 
     def coerce(self, obj, attr, value):
-        value = six.text_type(value)
+        value = str(value)
         if value not in self._valid_values:
             raise ValueError(_("Value '%(value)s' is not acceptable for "
                                "field '%(attr)s'.") %
@@ -519,3 +518,17 @@ class ReceiverTypeField(fields.AutoTypedField):
 class NodeReplaceMapField(fields.AutoTypedField):
 
     AUTO_TYPE = UniqueDict(fields.String())
+
+
+class CustomListField(ListField):
+
+    def __init__(self, attr_name, **kwargs):
+        self.attr_name = attr_name
+        super(CustomListField, self).__init__(**kwargs)
+
+    def coerce(self, obj, attr, value):
+        objs = super(CustomListField, self).coerce(obj, attr, value)
+        custom_list = []
+        for i in objs:
+            custom_list.append(getattr(i, self.attr_name))
+        return custom_list
