@@ -25,14 +25,17 @@ from senlin.objects import credential as co
 from senlin.objects import policy as po
 
 CHECK_RESULTS = (
-    CHECK_OK, CHECK_ERROR,
+    CHECK_OK, CHECK_ERROR, CHECK_NONE
 ) = (
-    'OK', 'ERROR',
+    'OK', 'ERROR', 'NONE'
 )
 
 
 class Policy(object):
-    '''Base class for policies.'''
+    """Base class for policies."""
+
+    VERSIONS = {}
+
     PROFILE_TYPE = 'ANY'
 
     KEYS = (
@@ -114,6 +117,7 @@ class Policy(object):
         self._novaclient = None
         self._keystoneclient = None
         self._networkclient = None
+        self._octaviaclient = None
         self._lbaasclient = None
 
     @classmethod
@@ -157,24 +161,12 @@ class Policy(object):
 
         return cls._from_object(db_policy)
 
-    @classmethod
-    def load_all(cls, context, limit=None, marker=None, sort=None,
-                 filters=None, project_safe=True):
-        """Retrieve all policies from database."""
-
-        objs = po.Policy.get_all(context, limit=limit, marker=marker,
-                                 sort=sort, filters=filters,
-                                 project_safe=project_safe)
-
-        for obj in objs:
-            yield cls._from_object(obj)
-
     @classmethod
     def delete(cls, context, policy_id):
         po.Policy.delete(context, policy_id)
 
     def store(self, context):
-        '''Store the policy object into database table.'''
+        """Store the policy object into database table."""
         timestamp = timeutils.utcnow(True)
 
         values = {
@@ -200,7 +192,7 @@ class Policy(object):
         return self.id
 
     def validate(self, context, validate_props=False):
-        '''Validate the schema and the data provided.'''
+        """Validate the schema and the data provided."""
         self.spec_data.validate()
         self.properties.validate()
 
@@ -234,14 +226,17 @@ class Policy(object):
         """Build trust-based connection parameters.
 
         :param user: the user for which the trust will be checked.
-        :param object: the user for which the trust will be checked.
+        :param project: the user for which the trust will be checked.
         """
-        service_creds = senlin_context.get_service_context()
+        service_creds = senlin_context.get_service_credentials()
         params = {
             'username': service_creds.get('username'),
             'password': service_creds.get('password'),
             'auth_url': service_creds.get('auth_url'),
-            'user_domain_name': service_creds.get('user_domain_name')
+            'user_domain_name': service_creds.get('user_domain_name'),
+            'project_domain_name': service_creds.get('project_domain_name'),
+            'verify': service_creds.get('verify'),
+            'interface': service_creds.get('interface'),
         }
 
         cred = co.Credential.get(oslo_context.get_current(), user, project)
@@ -292,6 +287,21 @@ class Policy(object):
         self._networkclient = driver.SenlinDriver().network(params)
         return self._networkclient
 
+    def octavia(self, user, project):
+        """Construct octavia client based on user and project.
+
+        :param user: The ID of the requesting user.
+        :param project: The ID of the requesting project.
+        :returns: A reference to the octavia client.
+        """
+        if self._octaviaclient is not None:
+            return self._octaviaclient
+
+        params = self._build_conn_params(user, project)
+        self._octaviaclient = driver.SenlinDriver().octavia(params)
+
+        return self._octaviaclient
+
     def lbaas(self, user, project):
         """Construct LB service client based on user and project.
 
@@ -307,13 +317,14 @@ class Policy(object):
         self._lbaasclient = driver.SenlinDriver().loadbalancing(params)
         return self._lbaasclient
 
-    def attach(self, cluster):
-        '''Method to be invoked before policy is attached to a cluster.
+    def attach(self, cluster, enabled=True):
+        """Method to be invoked before policy is attached to a cluster.
 
-        :param cluster: the cluster to which the policy is being attached to.
+        :param cluster: The cluster to which the policy is being attached to.
+        :param enabled: The attached cluster policy is enabled or disabled.
         :returns: (True, message) if the operation is successful, or (False,
                  error) otherwise.
-        '''
+        """
         if self.PROFILE_TYPE == ['ANY']:
             return True, None
 
@@ -326,7 +337,7 @@ class Policy(object):
         return True, None
 
     def detach(self, cluster):
-        '''Method to be invoked before policy is detached from a cluster.'''
+        """Method to be invoked before policy is detached from a cluster."""
         return True, None
 
     def need_check(self, target, action):
@@ -339,11 +350,11 @@ class Policy(object):
             return False
 
     def pre_op(self, cluster_id, action):
-        '''A method that will be invoked before an action execution.'''
+        """A method that will be invoked before an action execution."""
         return
 
     def post_op(self, cluster_id, action):
-        '''A method that will be invoked after an action execution.'''
+        """A method that will be invoked after an action execution."""
         return
 
     def to_dict(self):
