@@ -80,10 +80,10 @@ class LoadBalancingPolicy(base.Policy):
 
     KEYS = (
         POOL, VIP, HEALTH_MONITOR, LB_STATUS_TIMEOUT, LOADBALANCER,
-        AVAILABILITY_ZONE
+        AVAILABILITY_ZONE, FLAVOR_ID,
     ) = (
         'pool', 'vip', 'health_monitor', 'lb_status_timeout', 'loadbalancer',
-        'availability_zone'
+        'availability_zone', 'flavor_id',
     )
 
     _POOL_KEYS = (
@@ -142,8 +142,9 @@ class LoadBalancingPolicy(base.Policy):
 
     PERSISTENCE_TYPES = (
         PERSIST_SOURCE_IP, PERSIST_HTTP_COOKIE, PERSIST_APP_COOKIE,
+        PERSIST_NONE,
     ) = (
-        'SOURCE_IP', 'HTTP_COOKIE', 'APP_COOKIE',
+        'SOURCE_IP', 'HTTP_COOKIE', 'APP_COOKIE', 'NONE',
     )
 
     properties_schema = {
@@ -301,6 +302,11 @@ class LoadBalancingPolicy(base.Policy):
             _('Name of the loadbalancer availability zone to use for creation '
               'of the loadbalancer.'),
             default=None,
+        ),
+        FLAVOR_ID: schema.String(
+            _('ID of octavia loadbalancer flavor to use for creation '
+              'of the loadbalancer.'),
+            default=None,
         )
     }
 
@@ -311,6 +317,7 @@ class LoadBalancingPolicy(base.Policy):
         self.vip_spec = self.properties.get(self.VIP, {})
         self.hm_spec = self.properties.get(self.HEALTH_MONITOR, None)
         self.az_spec = self.properties.get(self.AVAILABILITY_ZONE, None)
+        self.flavor_id_spec = self.properties.get(self.FLAVOR_ID, None)
         self.lb_status_timeout = self.properties.get(self.LB_STATUS_TIMEOUT)
         self.lb = self.properties.get(self.LOADBALANCER, None)
 
@@ -332,6 +339,16 @@ class LoadBalancingPolicy(base.Policy):
                     ) % {'key': self.POOL_SUBNET, 'value': name_or_id}
             raise exc.InvalidSpec(message=msg)
 
+        # validate loadbalancer flavor_id
+        flavor_id = self.flavor_id_spec
+        if flavor_id:
+            try:
+                oc.find_flavor(flavor_id)
+            except exc.InternalError:
+                msg = _("The specified %(key)s '%(value)s' could not be found."
+                        ) % {'key': self.FLAVOR_ID, 'value': flavor_id}
+                raise exc.InvalidSpec(message=msg)
+
         # validate VIP subnet or network
         subnet_name_or_id = self.vip_spec.get(self.VIP_SUBNET)
         network_name_or_id = self.vip_spec.get(self.VIP_NETWORK)
@@ -385,6 +402,9 @@ class LoadBalancingPolicy(base.Policy):
         lb_driver = self.lbaas(cluster.user, cluster.project)
         lb_driver.lb_status_timeout = self.lb_status_timeout
 
+        # Set default name variable senlin cluster name
+        cluster_name = cluster.name
+
         # TODO(Anyone): Check if existing nodes has conflicts regarding the
         # subnets. Each VM addresses detail has a key named to the network
         # which can be used for validation.
@@ -398,7 +418,9 @@ class LoadBalancingPolicy(base.Policy):
                 data['healthmonitor'] = self.hm_spec.get(self.HM_ID)
         else:
             res, data = lb_driver.lb_create(self.vip_spec, self.pool_spec,
-                                            self.hm_spec, self.az_spec)
+                                            cluster_name,
+                                            self.hm_spec, self.az_spec,
+                                            self.flavor_id_spec)
             if res is False:
                 return False, data
 
