@@ -11,13 +11,14 @@
 # under the License.
 
 import copy
+import eventlet
 import inspect
 
+from oslo_config import cfg
 from oslo_context import context as oslo_context
 from oslo_log import log as logging
 from oslo_utils import timeutils
 from osprofiler import profiler
-import six
 
 from senlin.common import consts
 from senlin.common import context
@@ -127,6 +128,7 @@ class Profile(object):
         self._orchestrationclient = None
         self._workflowclient = None
         self._block_storageclient = None
+        self._glanceclient = None
 
     @classmethod
     def _from_object(cls, profile):
@@ -179,7 +181,7 @@ class Profile(object):
             profile.validate(True)
         except (exc.ResourceNotFound, exc.ESchema) as ex:
             error = _("Failed in creating profile %(name)s: %(error)s"
-                      ) % {"name": name, "error": six.text_type(ex)}
+                      ) % {"name": name, "error": str(ex)}
             raise exc.InvalidSpec(message=error)
 
         profile.store(ctx)
@@ -299,11 +301,13 @@ class Profile(object):
     @profiler.trace('Profile.check_object', hide_args=False)
     def check_object(cls, ctx, obj):
         profile = cls.load(ctx, profile_id=obj.profile_id)
-        try:
-            return profile.do_check(obj)
-        except exc.InternalError as ex:
-            LOG.error(ex)
-            return False
+        return profile.do_check(obj)
+
+    @classmethod
+    @profiler.trace('Profile.check_object', hide_args=False)
+    def healthcheck_object(cls, ctx, obj, health_check_type):
+        profile = cls.load(ctx, profile_id=obj.profile_id)
+        return profile.do_healthcheck(obj, health_check_type)
 
     @classmethod
     @profiler.trace('Profile.recover_object', hide_args=False)
@@ -319,7 +323,7 @@ class Profile(object):
 
         ctx_dict = self.properties.get('context', {})
         if ctx_dict:
-            argspec = inspect.getargspec(context.RequestContext.__init__)
+            argspec = inspect.getfullargspec(context.RequestContext.__init__)
             valid_keys = argspec.args
             bad_keys = [k for k in ctx_dict if k not in valid_keys]
             if bad_keys:
@@ -385,6 +389,19 @@ class Profile(object):
         self._computeclient = driver_base.SenlinDriver().compute(params)
         return self._computeclient
 
+    def glance(self, obj):
+        """Construct glance client based on object.
+
+        :param obj: Object for which the client is created. It is expected to
+                    be None when retrieving an existing client. When creating
+                    a client, it contains the user and project to be used.
+        """
+        if self._glanceclient is not None:
+            return self._glanceclient
+        params = self._build_conn_params(obj.user, obj.project)
+        self._glanceclient = driver_base.SenlinDriver().glance(params)
+        return self._glanceclient
+
     def network(self, obj):
         """Construct network client based on object.
 
@@ -459,6 +476,18 @@ class Profile(object):
         LOG.warning("Check operation not supported.")
         return True
 
+    def do_healthcheck(self, obj):
+        """Default healthcheck operation.
+
+        This is provided as a fallback if a specific profile type does not
+        override this method.
+
+        :param obj: The node object to operate on.
+        :return status: True indicates node is healthy, False indicates
+            it is unhealthy.
+        """
+        return self.do_check(obj)
+
     def do_get_details(self, obj):
         """For subclass to override."""
         LOG.warning("Get_details operation not supported.")
@@ -492,23 +521,21 @@ class Profile(object):
         :return status: True indicates successful recovery, False indicates
             failure.
         """
-        operation = options.pop('operation', None)
-        force_recreate = options.pop('force_recreate', None)
-        delete_timeout = options.pop('delete_timeout', None)
+        operation = options.get('operation', None)
+        force_recreate = options.get('force_recreate', None)
+        delete_timeout = options.get('delete_timeout', None)
 
-        # The operation is a list of action names with optional parameters
-        if operation and not isinstance(operation, six.string_types):
-            operation = operation[0]
-
-        if operation and operation['name'] != consts.RECOVER_RECREATE:
+        if operation.upper() != consts.RECOVER_RECREATE:
             LOG.error("Recover operation not supported: %s", operation)
             return None, False
 
-        extra_params = options.get('params', {})
-        fence_compute = extra_params.get('fence_compute', False)
+        extra_params = options.get('operation_params', None)
+        fence_compute = False
+        if extra_params:
+            fence_compute = extra_params.get('fence_compute', False)
+
         try:
-            self.do_delete(obj, force=fence_compute, timeout=delete_timeout,
-                           delete_ports_on_failure=force_recreate)
+            self.do_delete(obj, force=fence_compute, timeout=delete_timeout)
         except exc.EResourceDeletion as ex:
             if force_recreate:
                 # log error and continue on to creating the node
@@ -517,13 +544,21 @@ class Profile(object):
             else:
                 raise exc.EResourceOperation(op='recovering', type='node',
                                              id=obj.id,
-                                             message=six.text_type(ex))
+                                             message=str(ex))
+
+        # pause to allow deleted resource to get reclaimed by nova
+        # this is needed to avoid a problem when the compute resources are
+        # at their quota limit.  The deleted resource has to become available
+        # so that the new node can be created.
+        eventlet.sleep(cfg.CONF.batch_interval)
+
         res = None
         try:
             res = self.do_create(obj)
         except exc.EResourceCreation as ex:
             raise exc.EResourceOperation(op='recovering', type='node',
-                                         id=obj.id, message=six.text_type(ex))
+                                         id=obj.id, message=str(ex),
+                                         resource_id=ex.resource_id)
         return res, True
 
     def do_validate(self, obj):
