@@ -64,10 +64,10 @@ class ServerProfile(base.Profile):
     )
 
     NETWORK_KEYS = (
-        PORT, FIXED_IP, NETWORK, PORT_SECURITY_GROUPS,
+        PORT, VNIC_TYPE, FIXED_IP, NETWORK, PORT_SECURITY_GROUPS,
         FLOATING_NETWORK, FLOATING_IP,
     ) = (
-        'port', 'fixed_ip', 'network', 'security_groups',
+        'port', 'vnic_type', 'fixed_ip', 'network', 'security_groups',
         'floating_network', 'floating_ip',
     )
 
@@ -182,6 +182,9 @@ class ServerProfile(base.Profile):
                     PORT: schema.String(
                         _('Port ID to be used by the network.'),
                     ),
+                    VNIC_TYPE: schema.String(
+                        _('Define vnic_type to be used by port'),
+                    ),
                     FIXED_IP: schema.String(
                         _('Fixed IP to be used by the network.'),
                     ),
@@ -331,6 +334,7 @@ class ServerProfile(base.Profile):
     def __init__(self, type_name, name, **kwargs):
         super(ServerProfile, self).__init__(type_name, name, **kwargs)
         self.server_id = None
+        self.stop_timeout = cfg.CONF.default_nova_timeout
 
     def _validate_az(self, obj, az_name, reason=None):
         try:
@@ -627,6 +631,15 @@ class ServerProfile(base.Profile):
                           ) % {'p': self.PORT, 'fip': self.FIXED_IP})
             result[self.FIXED_IP] = fixed_ip
 
+        # Validate vnic_type input
+        vnic_type = net_spec.get(self.VNIC_TYPE, None)
+        if vnic_type is not None:
+            if vnic_type not in ['normal', 'direct', 'macvtap']:
+                _verify(_("vnic_type: '%(v)s' is not supported."
+                          "(supported types are: normal, direct, macvtap)"
+                          ) % {'v': vnic_type})
+            result[self.VNIC_TYPE] = vnic_type
+
         # Check security_groups
         error = self._check_security_groups(nc, net_spec, result)
         _verify(error)
@@ -660,8 +673,12 @@ class ServerProfile(base.Profile):
         security_groups = net_spec.get(self.PORT_SECURITY_GROUPS, [])
         if security_groups:
             port_attr['security_groups'] = security_groups
+        vnic_type = net_spec.get(self.VNIC_TYPE, None)
+        if vnic_type:
+            port_attr['binding_vnic_type'] = vnic_type
         try:
             port = self.network(obj).port_create(**port_attr)
+            LOG.debug('Network port_attr : %s', port)
             return port, None
         except exc.InternalError as ex:
             return None, ex
@@ -1106,7 +1123,7 @@ class ServerProfile(base.Profile):
         :param obj: The node object to operate on.
         :param old_flavor: The identity of the current flavor.
         :param new_flavor: The identity of the new flavor.
-        :returns: ``None``.
+        :returns: Returns true if the flavor was updated or false otherwise.
         :raises: `EResourceUpdate` when operation was a failure.
         """
         old_flavor = self.properties[self.FLAVOR]
@@ -1115,7 +1132,23 @@ class ServerProfile(base.Profile):
         oldflavor = self._validate_flavor(obj, old_flavor, 'update')
         newflavor = self._validate_flavor(obj, new_flavor, 'update')
         if oldflavor.id == newflavor.id:
-            return
+            return False
+
+        try:
+            # server has to be active or stopped in order to resize
+            # stop server if it is active
+            server = cc.server_get(obj.physical_id)
+            if server.status == consts.VS_ACTIVE:
+                cc.server_stop(obj.physical_id)
+                cc.wait_for_server(obj.physical_id, consts.VS_SHUTOFF,
+                                   timeout=self.stop_timeout)
+            elif server.status != consts.VS_SHUTOFF:
+                raise exc.InternalError(
+                    message='Server needs to be ACTIVE or STOPPED in order to'
+                            ' update flavor.')
+        except exc.InternalError as ex:
+            raise exc.EResourceUpdate(type='server', id=obj.physical_id,
+                                      message=str(ex))
 
         try:
             cc.server_resize(obj.physical_id, newflavor.id)
@@ -1123,7 +1156,13 @@ class ServerProfile(base.Profile):
         except exc.InternalError as ex:
             msg = str(ex)
             try:
-                cc.server_resize_revert(obj.physical_id)
+                server = cc.server_get(obj.physical_id)
+                if server.status == 'RESIZE':
+                    cc.server_resize_revert(obj.physical_id)
+                    cc.wait_for_server(obj.physical_id, consts.VS_SHUTOFF)
+
+                # start server back up in case of exception during resize
+                cc.server_start(obj.physical_id)
                 cc.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
             except exc.InternalError as ex1:
                 msg = str(ex1)
@@ -1132,11 +1171,13 @@ class ServerProfile(base.Profile):
 
         try:
             cc.server_resize_confirm(obj.physical_id)
-            cc.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
+            cc.wait_for_server(obj.physical_id, consts.VS_SHUTOFF)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
                                       message=str(ex))
 
+        return True
+
     def _update_image(self, obj, new_profile, new_name, new_password):
         """Update image used by server node.
 
@@ -1172,9 +1213,20 @@ class ServerProfile(base.Profile):
             return False
 
         try:
+            # server has to be active or stopped in order to resize
+            # stop server if it is active
+            if server.status == consts.VS_ACTIVE:
+                driver.server_stop(obj.physical_id)
+                driver.wait_for_server(obj.physical_id, consts.VS_SHUTOFF,
+                                       timeout=self.stop_timeout)
+            elif server.status != consts.VS_SHUTOFF:
+                raise exc.InternalError(
+                    message='Server needs to be ACTIVE or STOPPED in order to'
+                            ' update image.')
+
             driver.server_rebuild(obj.physical_id, new_image_id,
                                   new_name, new_password)
-            driver.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
+            driver.wait_for_server(obj.physical_id, consts.VS_SHUTOFF)
         except exc.InternalError as ex:
             raise exc.EResourceUpdate(type='server', id=obj.physical_id,
                                       message=str(ex))
@@ -1262,6 +1314,18 @@ class ServerProfile(base.Profile):
         nc = self.network(obj)
         internal_ports = obj.data.get('internal_ports', [])
 
+        if networks:
+            try:
+                # stop server if it is active
+                server = cc.server_get(obj.physical_id)
+                if server.status == consts.VS_ACTIVE:
+                    cc.server_stop(obj.physical_id)
+                    cc.wait_for_server(obj.physical_id, consts.VS_SHUTOFF,
+                                       timeout=self.stop_timeout)
+            except exc.InternalError as ex:
+                raise exc.EResourceUpdate(type='server', id=obj.physical_id,
+                                          message=str(ex))
+
         for n in networks:
             candidate_ports = self._find_port_by_net_spec(
                 obj, n, internal_ports)
@@ -1284,22 +1348,85 @@ class ServerProfile(base.Profile):
         obj.data['internal_ports'] = internal_ports
         node_obj.Node.update(self.context, obj.id, {'data': obj.data})
 
+    def _nw_compare(self, n1, n2, property):
+        return n1.get(property, None) == n2.get(property, None)
+
+    def _update_network_update_port(self, obj, networks):
+        """Update existing port in network from the node.
+
+           Currently only update to security group is supported.
+
+        :param obj: The node object to operate.
+        :param networks: A list networks that contain updated security groups.
+        :returns: ``None``
+        :raises: ``EResourceUpdate``
+        """
+        nc = self.network(obj)
+        internal_ports = obj.data.get('internal_ports', [])
+
+        # process each network to be updated
+        for n in networks:
+            # verify network properties and resolve names into ids
+            net = self._validate_network(obj, n, 'update')
+
+            # find existing port that matches network
+            candidate_ports = self._find_port_by_net_spec(
+                obj, net, internal_ports)
+            port = candidate_ports[0]
+            try:
+                # set updated security groups for port
+                port_attr = {
+                    'security_groups': net.get(self.PORT_SECURITY_GROUPS, []),
+                }
+                LOG.debug("Setting security groups %s for port %s",
+                          port_attr, port['id'])
+                nc.port_update(port['id'], **port_attr)
+            except exc.InternalError as ex:
+                raise exc.EResourceUpdate(type='server', id=obj.physical_id,
+                                          message=str(ex))
+
     def _update_network(self, obj, new_profile):
         """Updating server network interfaces.
 
         :param obj: The node object to operate.
         :param new_profile: The new profile which may contain new network
                             settings.
-        :return: ``None``
+        :return: Returns a tuple of booleans if network was created or deleted.
         :raises: ``EResourceUpdate`` if there are driver failures.
         """
         networks_current = self.properties[self.NETWORKS]
         networks_create = new_profile.properties[self.NETWORKS]
         networks_delete = copy.deepcopy(networks_current)
-        for network in networks_current:
-            if network in networks_create:
-                networks_create.remove(network)
-                networks_delete.remove(network)
+        networks_update = []
+
+        for nw in networks_current:
+            if nw in networks_create:
+                # network already exist.  no need to create or delete it.
+                LOG.debug("Network %s already exists, skip create/delete", nw)
+                networks_create.remove(nw)
+                networks_delete.remove(nw)
+
+        # find networks for which only security group changed
+        for nw in networks_current:
+            # networks to be created with only sg changes
+            sg_create_nw = [n for n in networks_create
+                            if (self._nw_compare(n, nw, 'network') and
+                                self._nw_compare(n, nw, 'port') and
+                                self._nw_compare(n, nw, 'fixed_ip') and
+                                self._nw_compare(n, nw, 'floating_network') and
+                                self._nw_compare(n, nw, 'floating_ip'))]
+            for n in sg_create_nw:
+                # don't create networks with only security group changes
+                LOG.debug("Network %s only has security group changes, "
+                          "don't create/delete it.  Only update it.", n)
+                networks_create.remove(n)
+                networks_update.append(n)
+                if nw in networks_delete:
+                    networks_delete.remove(nw)
+
+        # update network
+        if networks_update:
+            self._update_network_update_port(obj, networks_update)
 
         # Detach some existing interfaces
         if networks_delete:
@@ -1308,7 +1435,8 @@ class ServerProfile(base.Profile):
         # Attach new interfaces
         if networks_create:
             self._update_network_add_port(obj, networks_create)
-        return
+
+        return networks_create, networks_delete
 
     def do_update(self, obj, new_profile=None, **params):
         """Perform update on the server.
@@ -1328,6 +1456,15 @@ class ServerProfile(base.Profile):
         if not self.validate_for_update(new_profile):
             return False
 
+        self.stop_timeout = params.get('cluster.stop_timeout_before_update',
+                                       cfg.CONF.default_nova_timeout)
+
+        if not isinstance(self.stop_timeout, int):
+            raise exc.EResourceUpdate(
+                type='server', id=obj.physical_id,
+                message='cluster.stop_timeout_before_update value must be of '
+                        'type int.')
+
         name_changed, new_name = self._check_server_name(obj, new_profile)
         passwd_changed, new_passwd = self._check_password(obj, new_profile)
         # Update server image: may have side effect of changing server name
@@ -1342,13 +1479,26 @@ class ServerProfile(base.Profile):
                 self._update_password(obj, new_passwd)
 
         # Update server flavor: note that flavor is a required property
-        self._update_flavor(obj, new_profile)
-        self._update_network(obj, new_profile)
+        flavor_changed = self._update_flavor(obj, new_profile)
+        network_created, network_deleted = self._update_network(
+            obj, new_profile)
 
         # TODO(Yanyan Hu): Update block_device properties
         # Update server metadata
         self._update_metadata(obj, new_profile)
 
+        # start server if it was stopped as part of this update operation
+        if image_changed or flavor_changed or network_deleted:
+            cc = self.compute(obj)
+            try:
+                server = cc.server_get(obj.physical_id)
+                if server.status == consts.VS_SHUTOFF:
+                    cc.server_start(obj.physical_id)
+                    cc.wait_for_server(obj.physical_id, consts.VS_ACTIVE)
+            except exc.InternalError as ex:
+                raise exc.EResourceUpdate(type='server', id=obj.physical_id,
+                                          message=str(ex))
+
         return True
 
     def do_get_details(self, obj):
@@ -1449,11 +1599,8 @@ class ServerProfile(base.Profile):
         :return: The flavor_id for the server.
         """
         flavor = server['flavor']
-
-        if 'id' in flavor:
-            return flavor['id']
-
-        return self.compute(obj).flavor_find(flavor['original_name'], False).id
+        flavor_name = flavor.get('name') or flavor.get('original_name')
+        return self.compute(obj).flavor_find(flavor_name, False).id
 
     def _get_image_id(self, obj, server, op):
         """Get image id.
@@ -1638,22 +1785,20 @@ class ServerProfile(base.Profile):
 
         return True
 
-    def do_healthcheck(self, obj):
+    def do_healthcheck(self, obj, health_check_type):
         """Healthcheck operation.
 
-        This method checks if a server node is healthy by getting the server
-        status from nova.  A server is considered unhealthy if it does not
-        exist or its status is one of the following:
-        - ERROR
-        - SHUTOFF
-        - DELETED
+        This method checks if a node is healthy.  If health check type is
+        NODE_STATUS_POLLING it will check the server status.  If health check
+        type is HYPERVISOR_STATUS_POLLING it will check the hypervisor state
+        and status.
 
         :param obj: The node object to operate on.
+        :param health_check_type: The type of health check.  Either
+        NODE_STATUS_POLLING or HYPERVISOR_STATUS_POLLING.
         :return status: True indicates node is healthy, False indicates
             it is unhealthy.
         """
-        unhealthy_server_status = [consts.VS_ERROR, consts.VS_SHUTOFF,
-                                   consts.VS_DELETED]
 
         if not obj.physical_id:
             if obj.status == 'BUILD' or obj.status == 'CREATING':
@@ -1687,6 +1832,34 @@ class ServerProfile(base.Profile):
                 consts.POLL_STATUS_PASS, obj.name)
             return True
 
+        if health_check_type == consts.NODE_STATUS_POLLING:
+            return self._do_healthcheck_server(obj, server)
+        elif health_check_type == consts.HYPERVISOR_STATUS_POLLING:
+            return self._do_healthcheck_hypervisor(obj, server)
+        else:
+            LOG.info('%s for %s: ignoring invalid health check type %s',
+                     consts.POLL_STATUS_PASS, obj.name, health_check_type)
+            return True
+
+    def _do_healthcheck_server(self, obj, server):
+        """Healthcheck operation based on server.
+
+        This method checks if a server node is healthy by getting the server
+        status from nova.  A server is considered unhealthy if it does not
+        exist or its status is one of the following:
+        - ERROR
+        - SHUTOFF
+        - DELETED
+
+        :param obj: The node object to operate on.
+        :param server: The server object associated with the node.
+        :return status: True indicates node is healthy, False indicates
+            it is unhealthy.
+        """
+
+        unhealthy_server_status = [consts.VS_ERROR, consts.VS_SHUTOFF,
+                                   consts.VS_DELETED]
+
         if server.status in unhealthy_server_status:
             LOG.info('%s for %s: server status is unhealthy.',
                      consts.POLL_STATUS_FAIL, obj.name)
@@ -1695,6 +1868,66 @@ class ServerProfile(base.Profile):
         LOG.info('%s for %s', consts.POLL_STATUS_PASS, obj.name)
         return True
 
+    def _do_healthcheck_hypervisor(self, obj, server):
+        """Healthcheck operation based on hypervisor.
+
+        This method checks if a server node is healthy by getting the
+        hypervisor state and status from nova.  A server is considered
+        unhealthy if the hypervisor it is running on has a state that is down
+        or a status that is disabled.
+
+        :param obj: The node object to operate on.
+        :param server: The server object associated with the node.
+        :return status: True indicates node is healthy, False indicates
+            it is unhealthy.
+        """
+
+        if server.hypervisor_hostname != "":
+            try:
+                hv = self.compute(obj).hypervisor_find(
+                    server.hypervisor_hostname)
+            except Exception as ex:
+                if isinstance(ex, exc.InternalError) and ex.code == 404:
+                    # treat resource not found exception as unhealthy
+                    LOG.info('%s for %s: hypervisor %s was not found.',
+                             consts.POLL_STATUS_FAIL, obj.name,
+                             server.hypervisor_hostname)
+                    return False
+                else:
+                    # treat all other exceptions as healthy
+                    LOG.info(
+                        '%s for %s: Exception when trying to get hypervisor '
+                        'info for %s, but ignoring this error: %s.',
+                        consts.POLL_STATUS_PASS, obj.name,
+                        server.hypervisor_hostname, ex.message)
+                    return True
+
+            if hv is None:
+                # no hypervisor information is available, treat the node as
+                # healthy
+                LOG.info(
+                    '%s for %s: No hypervisor information was returned but '
+                    'ignoring this error.',
+                    consts.POLL_STATUS_PASS, obj.name)
+                return True
+
+            if hv.state == 'down':
+                LOG.info('%s for %s: server status is unhealthy because '
+                         'hypervisor %s state is down',
+                         consts.POLL_STATUS_FAIL, obj.name,
+                         server.hypervisor_hostname)
+                return False
+
+            if hv.status == 'disabled':
+                LOG.info('%s for %s: server status is unhealthy because '
+                         'hypervisor %s status is disabled',
+                         consts.POLL_STATUS_FAIL, obj.name,
+                         server.hypervisor_hostname)
+                return False
+
+        LOG.info('%s for %s', consts.POLL_STATUS_PASS, obj.name)
+        return True
+
     def do_recover(self, obj, **options):
         """Handler for recover operation.
 
