@@ -10,11 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
+
 from oslo_db.sqlalchemy import utils as sa_utils
 from oslo_serialization import jsonutils
 from oslo_utils import timeutils as tu
-import six
 
 from senlin.common import consts
 from senlin.common import exception
@@ -79,7 +79,8 @@ class DBAPINodeTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a_different_project',
                                         is_admin=True)
         node = db_api.node_get(admin_ctx, res.id, project_safe=True)
-        self.assertIsNone(node)
+        self.assertIsNotNone(node)
+
         node = db_api.node_get(admin_ctx, res.id, project_safe=False)
         self.assertIsNotNone(node)
 
@@ -154,7 +155,8 @@ class DBAPINodeTest(base.SenlinTestCase):
                                         is_admin=True)
         res = db_api.node_get_by_short_id(admin_ctx, node_id[:11],
                                           project_safe=True)
-        self.assertIsNone(res)
+        self.assertIsNotNone(res)
+
         res = db_api.node_get_by_short_id(admin_ctx, node_id[:11],
                                           project_safe=False)
         self.assertIsNotNone(res)
@@ -369,7 +371,8 @@ class DBAPINodeTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a_different_project',
                                         is_admin=True)
         results = db_api.node_get_all(admin_ctx, project_safe=True)
-        self.assertEqual(0, len(results))
+        self.assertEqual(2, len(results))
+
         results = db_api.node_get_all(admin_ctx, project_safe=False)
         self.assertEqual(2, len(results))
 
@@ -445,7 +448,8 @@ class DBAPINodeTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a_different_project',
                                         is_admin=True)
         nodes = db_api.node_get_all_by_cluster(admin_ctx, self.cluster.id)
-        self.assertEqual(0, len(nodes))
+        self.assertEqual(2, len(nodes))
+
         nodes = db_api.node_get_all_by_cluster(admin_ctx, self.cluster.id,
                                                project_safe=False)
         self.assertEqual(2, len(nodes))
@@ -499,7 +503,8 @@ class DBAPINodeTest(base.SenlinTestCase):
                                         is_admin=True)
         res = db_api.node_count_by_cluster(admin_ctx, self.cluster.id,
                                            project_safe=True)
-        self.assertEqual(0, res)
+        self.assertEqual(2, res)
+
         res = db_api.node_count_by_cluster(admin_ctx, self.cluster.id,
                                            project_safe=False)
         self.assertEqual(2, res)
@@ -555,7 +560,7 @@ class DBAPINodeTest(base.SenlinTestCase):
                                db_api.node_update,
                                self.ctx, 'BogusId', new_attributes)
         self.assertEqual("The node 'BogusId' could not be found.",
-                         six.text_type(ex))
+                         str(ex))
 
     def test_node_update_cluster_status_updated(self):
         cluster = db_api.cluster_get(self.ctx, self.cluster.id)
