@@ -10,7 +10,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
+
 from oslo_db.sqlalchemy import utils as sa_utils
 from oslo_utils import timeutils as tu
 
@@ -39,9 +40,9 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         data = {
             'name': 'test_policy',
             'type': 'ScalingPolicy',
-            'user': self.ctx.user,
-            'project': self.ctx.project,
-            'domain': self.ctx.domain,
+            'user': self.ctx.user_id,
+            'project': self.ctx.project_id,
+            'domain': self.ctx.domain_id,
             'spec': sample_spec,
             'data': None,
         }
@@ -92,6 +93,9 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         res = db_api.policy_get(admin_ctx, policy.id, project_safe=True)
         self.assertIsNotNone(res)
 
+        res = db_api.policy_get(admin_ctx, policy.id, project_safe=False)
+        self.assertIsNotNone(res)
+
     def test_policy_get_not_found(self):
         retobj = db_api.policy_get(self.ctx, 'BogusID')
         self.assertIsNone(retobj)
@@ -115,6 +119,12 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         retobj = db_api.policy_get_by_name(self.ctx, 'non-exist')
         self.assertIsNone(retobj)
 
+        # duplicated name
+        db_api.policy_create(self.ctx, data)
+        self.assertRaises(exception.MultipleChoices,
+                          db_api.policy_get_by_name,
+                          self.ctx, policy_name)
+
     def test_policy_get_by_name_diff_project(self):
         policy_name = 'my_best_policy'
         data = self.new_policy_data(name=policy_name)
@@ -227,7 +237,8 @@ class DBAPIPolicyTest(base.SenlinTestCase):
         admin_ctx = utils.dummy_context(project='a-different-project',
                                         is_admin=True)
         policies = db_api.policy_get_all(admin_ctx, project_safe=True)
-        self.assertEqual(0, len(policies))
+        self.assertEqual(2, len(policies))
+
         policies = db_api.policy_get_all(admin_ctx, project_safe=False)
         self.assertEqual(2, len(policies))
 
