@@ -11,8 +11,9 @@
 # under the License.
 
 import eventlet
-import mock
+from unittest import mock
 
+from senlin.common import consts
 from senlin.common import scaleutils
 from senlin.engine.actions import base as base_action
 from senlin.engine.actions import node_action
@@ -35,7 +36,7 @@ class NodeActionTest(base.SenlinTestCase):
 
     def test_do_create_okay(self, mock_load):
         node = mock.Mock(id='NID')
-        node.do_create = mock.Mock(return_value=True)
+        node.do_create = mock.Mock(return_value=[True, ''])
         mock_load.return_value = node
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx)
 
@@ -47,7 +48,8 @@ class NodeActionTest(base.SenlinTestCase):
 
     def test_do_create_failed(self, mock_load):
         node = mock.Mock(id='NID')
-        node.do_create = mock.Mock(return_value=False)
+        node.do_create = mock.Mock(return_value=[False,
+                                                 'custom error message'])
         mock_load.return_value = node
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx)
 
@@ -55,7 +57,7 @@ class NodeActionTest(base.SenlinTestCase):
         res_code, res_msg = action.do_create()
 
         self.assertEqual(action.RES_ERROR, res_code)
-        self.assertEqual('Node creation failed.', res_msg)
+        self.assertEqual('custom error message', res_msg)
         node.do_create.assert_called_once_with(action.context)
 
     @mock.patch.object(scaleutils, 'check_size_params')
@@ -66,12 +68,12 @@ class NodeActionTest(base.SenlinTestCase):
         cluster = mock.Mock(id='CID')
         mock_c_load.return_value = cluster
         node = mock.Mock(id='NID', cluster_id='CID')
-        node.do_create = mock.Mock(return_value=True)
+        node.do_create = mock.Mock(return_value=[True, ''])
         mock_load.return_value = node
-        mock_count.return_value = 10
+        mock_count.return_value = 11
         mock_check.return_value = None
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
-                                        cause=base_action.CAUSE_RPC)
+                                        cause=consts.CAUSE_RPC)
 
         # do it
         res_code, res_msg = action.do_create()
@@ -83,7 +85,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_check.assert_called_once_with(cluster, 11, None, None, True)
         node.do_create.assert_called_once_with(action.context)
         cluster.eval_status.assert_called_once_with(
-            action.context, action.NODE_CREATE, desired_capacity=11)
+            action.context, consts.NODE_CREATE, desired_capacity=11)
 
     @mock.patch.object(node_obj.Node, 'update')
     @mock.patch.object(scaleutils, 'check_size_params')
@@ -95,12 +97,12 @@ class NodeActionTest(base.SenlinTestCase):
         cluster = mock.Mock(id='CID')
         mock_c_load.return_value = cluster
         node = mock.Mock(id='NID', cluster_id='CID')
-        node.do_create = mock.Mock(return_value=True)
+        node.do_create = mock.Mock(return_value=[True, ''])
         mock_load.return_value = node
-        mock_count.return_value = 10
+        mock_count.return_value = 11
         mock_check.return_value = 'overflow'
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
-                                        cause=base_action.CAUSE_RPC)
+                                        cause=consts.CAUSE_RPC)
 
         # do it
         res_code, res_msg = action.do_create()
@@ -112,7 +114,8 @@ class NodeActionTest(base.SenlinTestCase):
         mock_count.assert_called_once_with(action.context, 'CID')
         mock_check.assert_called_once_with(cluster, 11, None, None, True)
         mock_update.assert_called_once_with(action.context, 'NID',
-                                            {'cluster_id': ''})
+                                            {'cluster_id': '',
+                                             'status': consts.NS_ERROR})
         self.assertEqual(0, node.do_create.call_count)
         self.assertEqual(0, cluster.eval_status.call_count)
 
@@ -125,25 +128,26 @@ class NodeActionTest(base.SenlinTestCase):
         cluster = mock.Mock(id='CID')
         mock_c_load.return_value = cluster
         node = mock.Mock(id='NID', cluster_id='CID')
-        node.do_create = mock.Mock(return_value=False)
+        node.do_create = mock.Mock(return_value=[False,
+                                                 'custom error message'])
         mock_load.return_value = node
-        mock_count.return_value = 10
+        mock_count.return_value = 11
         mock_check.return_value = ''
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
-                                        cause=base_action.CAUSE_RPC)
+                                        cause=consts.CAUSE_RPC)
 
         # do it
         res_code, res_msg = action.do_create()
 
         # assertions
         self.assertEqual(action.RES_ERROR, res_code)
-        self.assertEqual('Node creation failed.', res_msg)
+        self.assertEqual('custom error message', res_msg)
         mock_c_load.assert_called_once_with(action.context, 'CID')
         mock_count.assert_called_once_with(action.context, 'CID')
         mock_check.assert_called_once_with(cluster, 11, None, None, True)
         node.do_create.assert_called_once_with(action.context)
         cluster.eval_status.assert_called_once_with(
-            action.context, action.NODE_CREATE, desired_capacity=11)
+            action.context, consts.NODE_CREATE, desired_capacity=11)
 
     def test_do_delete_okay(self, mock_load):
         node = mock.Mock(id='NID')
@@ -185,7 +189,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_count.return_value = 2
         mock_check.return_value = None
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
-                                        cause=base_action.CAUSE_RPC)
+                                        cause=consts.CAUSE_RPC)
 
         # do it
         res_code, res_msg = action.do_delete()
@@ -197,7 +201,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_count.assert_called_once_with(action.context, 'CID')
         mock_check.assert_called_once_with(cluster, 1, None, None, True)
         cluster.eval_status.assert_called_once_with(
-            action.context, action.NODE_DELETE, desired_capacity=1)
+            action.context, consts.NODE_DELETE, desired_capacity=1)
 
     @mock.patch.object(scaleutils, 'check_size_params')
     @mock.patch.object(node_obj.Node, 'count_by_cluster')
@@ -213,7 +217,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_count.return_value = 2
         mock_check.return_value = 'underflow'
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
-                                        cause=base_action.CAUSE_RPC)
+                                        cause=consts.CAUSE_RPC)
 
         res_code, res_msg = action.do_delete()
 
@@ -241,7 +245,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_count.return_value = 2
         mock_check.return_value = None
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
-                                        cause=base_action.CAUSE_RPC)
+                                        cause=consts.CAUSE_RPC)
 
         res_code, res_msg = action.do_delete()
 
@@ -253,7 +257,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_check.assert_called_once_with(cluster, 1, None, None, True)
         node.do_delete.assert_called_once_with(action.context)
         cluster.eval_status.assert_called_once_with(
-            action.context, action.NODE_DELETE)
+            action.context, consts.NODE_DELETE)
 
     @mock.patch.object(eventlet, 'sleep')
     @mock.patch.object(scaleutils, 'check_size_params')
@@ -270,7 +274,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_count.return_value = 2
         mock_check.return_value = None
         action = node_action.NodeAction(
-            node.id, 'ACTION', self.ctx, cause=base_action.CAUSE_RPC,
+            node.id, 'ACTION', self.ctx, cause=consts.CAUSE_RPC,
             data={'deletion': {'grace_period': 10}})
 
         # do it
@@ -286,7 +290,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_sleep.assert_called_once_with(10)
         node.do_delete.assert_called_once_with(action.context)
         cluster.eval_status.assert_called_once_with(
-            action.context, action.NODE_DELETE, desired_capacity=1)
+            action.context, consts.NODE_DELETE, desired_capacity=1)
 
     @mock.patch.object(eventlet, 'sleep')
     @mock.patch.object(scaleutils, 'check_size_params')
@@ -303,7 +307,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_check.return_value = None
         action = node_action.NodeAction(
             'NID', 'ACTION', self.ctx,
-            cause=base_action.CAUSE_RPC,
+            cause=consts.CAUSE_RPC,
             data={'deletion': {'reduce_desired_capacity': True}})
 
         # do it
@@ -317,7 +321,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_check.assert_called_once_with(cluster, 1, None, None, True)
         node.do_delete.assert_called_once_with(action.context)
         cluster.eval_status.assert_called_once_with(
-            action.context, action.NODE_DELETE, desired_capacity=1)
+            action.context, consts.NODE_DELETE, desired_capacity=1)
 
     @mock.patch.object(eventlet, 'sleep')
     @mock.patch.object(scaleutils, 'check_size_params')
@@ -334,7 +338,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_check.return_value = None
         action = node_action.NodeAction(
             'NID', 'ACTION', self.ctx,
-            cause=base_action.CAUSE_RPC,
+            cause=consts.CAUSE_RPC,
             data={'deletion': {'reduce_desired_capacity': False}})
 
         # do it
@@ -348,7 +352,7 @@ class NodeActionTest(base.SenlinTestCase):
         mock_check.assert_called_once_with(cluster, 1, None, None, True)
         node.do_delete.assert_called_once_with(action.context)
         cluster.eval_status.assert_called_once_with(
-            action.context, action.NODE_DELETE)
+            action.context, consts.NODE_DELETE)
 
     def test_do_delete_derived_success(self, mock_load):
 
@@ -356,7 +360,7 @@ class NodeActionTest(base.SenlinTestCase):
         node.do_delete.return_value = True
         mock_load.return_value = node
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
-                                        cause=base_action.CAUSE_DERIVED)
+                                        cause=consts.CAUSE_DERIVED)
 
         res_code, res_msg = action.do_delete()
 
@@ -370,7 +374,7 @@ class NodeActionTest(base.SenlinTestCase):
         node.do_delete.return_value = False
         mock_load.return_value = node
         action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
-                                        cause=base_action.CAUSE_DERIVED)
+                                        cause=consts.CAUSE_DERIVED)
 
         res_code, res_msg = action.do_delete()
 
@@ -401,20 +405,28 @@ class NodeActionTest(base.SenlinTestCase):
         self.assertEqual('Node updated successfully.', res_msg)
         node.do_update.assert_called_once_with(action.context, inputs)
 
-    @mock.patch.object(scaleutils, 'check_size_params')
-    @mock.patch.object(node_obj.Node, 'count_by_cluster')
-    @mock.patch.object(cluster_mod.Cluster, 'load')
-    def test_do_join_success(self, mock_c_load, mock_count, mock_check,
-                             mock_load):
+    def test_do_update_no_need_update(self, mock_load):
+        node = mock.Mock()
+        node.id = 'NID'
+        node.profile_id = 'PROFILE_ID'
+        mock_load.return_value = node
+        inputs = {"new_profile_id": "PROFILE_ID"}
+        action = node_action.NodeAction(node.id, 'ACTION', self.ctx,
+                                        inputs=inputs)
+
+        # Test node update success path
+        node.do_update = mock.Mock(return_value=mock.Mock())
+        res_code, res_msg = action.do_update()
+        self.assertEqual(action.RES_OK, res_code)
+        self.assertEqual('No property to update.', res_msg)
+        self.assertFalse(node.do_update.called)
+
+    def test_do_join_success(self, mock_load):
         node = mock.Mock(id='NID')
         mock_load.return_value = node
         inputs = {"cluster_id": "FAKE_ID"}
         action = node_action.NodeAction(node.id, 'NODE_JOIN', self.ctx,
                                         inputs=inputs)
-        cluster = mock.Mock()
-        mock_c_load.return_value = cluster
-        mock_count.return_value = 3
-        mock_check.return_value = None
         node.do_join = mock.Mock(return_value=True)
 
         # Test failed node join path
@@ -422,51 +434,14 @@ class NodeActionTest(base.SenlinTestCase):
 
         self.assertEqual(action.RES_OK, res_code)
         self.assertEqual('Node successfully joined cluster.', res_msg)
-        mock_c_load.assert_called_once_with(action.context, 'FAKE_ID')
-        mock_count.assert_called_once_with(action.context, 'FAKE_ID')
-        mock_check.assert_called_once_with(cluster, 4, None, None, True)
         node.do_join.assert_called_once_with(action.context, 'FAKE_ID')
 
-    @mock.patch.object(scaleutils, 'check_size_params')
-    @mock.patch.object(node_obj.Node, 'count_by_cluster')
-    @mock.patch.object(cluster_mod.Cluster, 'load')
-    def test_do_join_failed_check(self, mock_c_load, mock_count, mock_check,
-                                  mock_load):
+    def test_do_join_failed_do_join(self, mock_load):
         node = mock.Mock(id='NID')
         mock_load.return_value = node
         inputs = {"cluster_id": "FAKE_ID"}
         action = node_action.NodeAction(node.id, 'NODE_JOIN', self.ctx,
                                         inputs=inputs)
-        cluster = mock.Mock()
-        mock_c_load.return_value = cluster
-        mock_count.return_value = 10
-        mock_check.return_value = 'Size limits'
-        node.do_join = mock.Mock(return_value=True)
-
-        # Test failed node join path
-        res_code, res_msg = action.do_join()
-
-        self.assertEqual(action.RES_ERROR, res_code)
-        self.assertEqual('Size limits', res_msg)
-        mock_c_load.assert_called_once_with(action.context, 'FAKE_ID')
-        mock_count.assert_called_once_with(action.context, 'FAKE_ID')
-        mock_check.assert_called_once_with(cluster, 11, None, None, True)
-        self.assertEqual(0, node.do_join.call_count)
-
-    @mock.patch.object(scaleutils, 'check_size_params')
-    @mock.patch.object(node_obj.Node, 'count_by_cluster')
-    @mock.patch.object(cluster_mod.Cluster, 'load')
-    def test_do_join_failed_do_join(self, mock_c_load, mock_count, mock_check,
-                                    mock_load):
-        node = mock.Mock(id='NID')
-        mock_load.return_value = node
-        inputs = {"cluster_id": "FAKE_ID"}
-        action = node_action.NodeAction(node.id, 'NODE_JOIN', self.ctx,
-                                        inputs=inputs)
-        cluster = mock.Mock()
-        mock_c_load.return_value = cluster
-        mock_count.return_value = 5
-        mock_check.return_value = None
         node.do_join = mock.Mock(return_value=False)
 
         # Test failed node join path
@@ -474,23 +449,12 @@ class NodeActionTest(base.SenlinTestCase):
 
         self.assertEqual(action.RES_ERROR, res_code)
         self.assertEqual('Node failed in joining cluster.', res_msg)
-        mock_c_load.assert_called_once_with(action.context, 'FAKE_ID')
-        mock_count.assert_called_once_with(action.context, 'FAKE_ID')
-        mock_check.assert_called_once_with(cluster, 6, None, None, True)
         node.do_join.assert_called_once_with(action.context, 'FAKE_ID')
 
-    @mock.patch.object(scaleutils, 'check_size_params')
-    @mock.patch.object(node_obj.Node, 'count_by_cluster')
-    @mock.patch.object(cluster_mod.Cluster, 'load')
-    def test_do_leave_success(self, mock_c_load, mock_count, mock_check,
-                              mock_load):
+    def test_do_leave_success(self, mock_load):
         node = mock.Mock(id='NID', cluster_id='CID')
         mock_load.return_value = node
         action = node_action.NodeAction(node.id, 'NODE_LEAVE', self.ctx)
-        cluster = mock.Mock(id='CID')
-        mock_c_load.return_value = cluster
-        mock_count.return_value = 10
-        mock_check.return_value = None
         node.do_leave = mock.Mock(return_value=True)
 
         # Test failed node join path
@@ -498,47 +462,12 @@ class NodeActionTest(base.SenlinTestCase):
 
         self.assertEqual(action.RES_OK, res_code)
         self.assertEqual('Node successfully left cluster.', res_msg)
-        mock_c_load.assert_called_once_with(action.context, 'CID')
-        mock_count.assert_called_once_with(action.context, 'CID')
-        mock_check.assert_called_once_with(cluster, 9, None, None, True)
         node.do_leave.assert_called_once_with(action.context)
 
-    @mock.patch.object(scaleutils, 'check_size_params')
-    @mock.patch.object(node_obj.Node, 'count_by_cluster')
-    @mock.patch.object(cluster_mod.Cluster, 'load')
-    def test_do_leave_failed_check(self, mock_c_load, mock_count, mock_check,
-                                   mock_load):
+    def test_do_leave_failed_leave(self, mock_load):
         node = mock.Mock(id='NID', cluster_id='CID')
         mock_load.return_value = node
         action = node_action.NodeAction(node.id, 'NODE_LEAVE', self.ctx)
-        cluster = mock.Mock()
-        mock_c_load.return_value = cluster
-        mock_count.return_value = 10
-        mock_check.return_value = 'Size limits'
-        node.do_leave = mock.Mock(return_value=True)
-
-        # Test failed node join path
-        res_code, res_msg = action.do_leave()
-
-        self.assertEqual(action.RES_ERROR, res_code)
-        self.assertEqual('Size limits', res_msg)
-        mock_c_load.assert_called_once_with(action.context, 'CID')
-        mock_count.assert_called_once_with(action.context, 'CID')
-        mock_check.assert_called_once_with(cluster, 9, None, None, True)
-        self.assertEqual(0, node.do_leave.call_count)
-
-    @mock.patch.object(scaleutils, 'check_size_params')
-    @mock.patch.object(node_obj.Node, 'count_by_cluster')
-    @mock.patch.object(cluster_mod.Cluster, 'load')
-    def test_do_leave_failed_leave(self, mock_c_load, mock_count, mock_check,
-                                   mock_load):
-        node = mock.Mock(id='NID', cluster_id='CID')
-        mock_load.return_value = node
-        action = node_action.NodeAction(node.id, 'NODE_LEAVE', self.ctx)
-        cluster = mock.Mock()
-        mock_c_load.return_value = cluster
-        mock_count.return_value = 10
-        mock_check.return_value = None
         node.do_leave = mock.Mock(return_value=False)
 
         # Test failed node join path
@@ -546,9 +475,6 @@ class NodeActionTest(base.SenlinTestCase):
 
         self.assertEqual(action.RES_ERROR, res_code)
         self.assertEqual('Node failed in leaving cluster.', res_msg)
-        mock_c_load.assert_called_once_with(action.context, 'CID')
-        mock_count.assert_called_once_with(action.context, 'CID')
-        mock_check.assert_called_once_with(cluster, 9, None, None, True)
         node.do_leave.assert_called_once_with(action.context)
 
     def test_do_check_success(self, mock_load):
@@ -560,7 +486,7 @@ class NodeActionTest(base.SenlinTestCase):
         res_code, res_msg = action.do_check()
 
         self.assertEqual(action.RES_OK, res_code)
-        self.assertEqual('Node status is ACTIVE.', res_msg)
+        self.assertEqual('Node check succeeded.', res_msg)
         node.do_check.assert_called_once_with(action.context)
 
     def test_do_check_failed(self, mock_load):
@@ -572,7 +498,7 @@ class NodeActionTest(base.SenlinTestCase):
         res_code, res_msg = action.do_check()
 
         self.assertEqual(action.RES_ERROR, res_code)
-        self.assertEqual('Node status is not ACTIVE.', res_msg)
+        self.assertEqual('Node check failed.', res_msg)
         node.do_check.assert_called_once_with(action.context)
 
     def test_do_recover_success(self, mock_load):
@@ -586,8 +512,7 @@ class NodeActionTest(base.SenlinTestCase):
 
         self.assertEqual(action.RES_OK, res_code)
         self.assertEqual('Node recovered successfully.', res_msg)
-        node.do_recover.assert_called_once_with(action.context,
-                                                operation=['SWIM', 'DANCE'])
+        node.do_recover.assert_called_once_with(action.context, action)
 
     def test_do_recover_failed(self, mock_load):
         node = mock.Mock(id='NID')
@@ -600,8 +525,37 @@ class NodeActionTest(base.SenlinTestCase):
         res_code, res_msg = action.do_recover()
         self.assertEqual(action.RES_ERROR, res_code)
         self.assertEqual('Node recover failed.', res_msg)
-        node.do_recover.assert_called_once_with(action.context,
-                                                operation=['SWIM', 'DANCE'])
+        node.do_recover.assert_called_once_with(action.context, action)
+
+    def test_do_operation_success(self, mock_load):
+        node = mock.Mock(id='NID')
+        mock_load.return_value = node
+        action = node_action.NodeAction(node.id, 'ACTION', self.ctx)
+        action.inputs = {'operation': 'dance', 'params': {}}
+        node.do_operation = mock.Mock(return_value=True)
+
+        res_code, res_msg = action.do_operation()
+
+        self.assertEqual(action.RES_OK, res_code)
+        self.assertEqual("Node operation 'dance' succeeded.", res_msg)
+        node.do_operation.assert_called_once_with(action.context,
+                                                  operation='dance',
+                                                  params={})
+
+    def test_do_operation_failed(self, mock_load):
+        node = mock.Mock(id='NID')
+        mock_load.return_value = node
+        action = node_action.NodeAction(node.id, 'ACTION', self.ctx)
+        action.inputs = {'operation': 'dance', 'params': {}}
+        node.do_operation = mock.Mock(return_value=False)
+
+        res_code, res_msg = action.do_operation()
+
+        self.assertEqual(action.RES_ERROR, res_code)
+        self.assertEqual("Node operation 'dance' failed.", res_msg)
+        node.do_operation.assert_called_once_with(action.context,
+                                                  operation='dance',
+                                                  params={})
 
     def test_execute(self, mock_load):
         node = mock.Mock()
@@ -628,9 +582,7 @@ class NodeActionTest(base.SenlinTestCase):
         self.assertEqual(action.RES_ERROR, res_code)
         reason = 'Unsupported action: NODE_DANCE'
         self.assertEqual(reason, res_msg)
-        mock_error.assert_called_once_with(action.context, node,
-                                           action.action, 'Failed',
-                                           reason)
+        mock_error.assert_called_once_with(action, 'error', reason)
 
     @mock.patch.object(lock, 'cluster_lock_acquire')
     def test_execute_failed_lock_cluster(self, mock_acquire, mock_load):
@@ -685,6 +637,40 @@ class NodeActionTest(base.SenlinTestCase):
                                              lock.NODE_SCOPE)
         mock_check.assert_called_once_with('FAKE_CLUSTER', 'BEFORE')
 
+    @mock.patch.object(lock, 'cluster_lock_acquire')
+    @mock.patch.object(lock, 'cluster_lock_release')
+    @mock.patch.object(base_action.Action, 'policy_check')
+    def test_execute_policy_check_exception(self, mock_check, mock_release,
+                                            mock_acquire, mock_load):
+        node = mock.Mock()
+        node.id = 'NID'
+        node.cluster_id = 'FAKE_CLUSTER'
+        mock_load.return_value = node
+
+        action = node_action.NodeAction('NODE_ID', 'NODE_FLY', self.ctx,
+                                        cause='RPC Request')
+        action.id = 'ACTION_ID'
+        action.data = {
+            'status': policy_mod.CHECK_NONE,
+            'reason': ''
+        }
+        mock_acquire.return_value = action.id
+
+        mock_check.side_effect = Exception('error')
+
+        res_code, res_msg = action.execute()
+
+        reason = 'Policy check: '
+        self.assertEqual(action.RES_ERROR, res_code)
+        self.assertEqual(reason, res_msg)
+        mock_load.assert_called_once_with(action.context, node_id='NODE_ID')
+        mock_acquire.assert_called_once_with(self.ctx, 'FAKE_CLUSTER',
+                                             'ACTION_ID', None,
+                                             lock.NODE_SCOPE, False)
+        mock_release.assert_called_once_with('FAKE_CLUSTER', 'ACTION_ID',
+                                             lock.NODE_SCOPE)
+        mock_check.assert_called_once_with('FAKE_CLUSTER', 'BEFORE')
+
     @mock.patch.object(lock, 'cluster_lock_acquire')
     @mock.patch.object(lock, 'cluster_lock_release')
     @mock.patch.object(lock, 'node_lock_acquire')
@@ -698,7 +684,7 @@ class NodeActionTest(base.SenlinTestCase):
         node = mock.Mock(id=node_id, cluster_id='FAKE_CLUSTER')
         mock_load.return_value = node
         action = node_action.NodeAction(node_id, 'NODE_FLY', self.ctx,
-                                        cause=base_action.CAUSE_DERIVED)
+                                        cause=consts.CAUSE_DERIVED)
         action.id = 'ACTION_ID'
         action.owner = 'OWNER'
         mock_exec = self.patchobject(action, '_execute',
@@ -746,7 +732,7 @@ class NodeActionTest(base.SenlinTestCase):
         res_code, res_msg = action.execute()
 
         reason = 'Failed in locking node'
-        self.assertEqual(action.RES_ERROR, res_code)
+        self.assertEqual(action.RES_RETRY, res_code)
         self.assertEqual(reason, res_msg)
         mock_load.assert_called_once_with(action.context, node_id='NODE_ID')
         mock_acquire.assert_called_once_with(self.ctx, 'FAKE_CLUSTER',
@@ -759,6 +745,52 @@ class NodeActionTest(base.SenlinTestCase):
                                                   'ACTION_ID', None, False)
         mock_release_node.assert_called_once_with('NODE_ID', 'ACTION_ID')
 
+    @mock.patch.object(lock, 'cluster_lock_acquire')
+    @mock.patch.object(lock, 'cluster_lock_release')
+    @mock.patch.object(base_action.Action, 'policy_check')
+    @mock.patch.object(lock, 'node_lock_acquire')
+    @mock.patch.object(lock, 'node_lock_release')
+    def test_execute_success_stealing_node_lock(self, mock_release_node,
+                                                mock_acquire_node, mock_check,
+                                                mock_release, mock_acquire,
+                                                mock_load):
+        node = mock.Mock()
+        node.cluster_id = 'FAKE_CLUSTER'
+        node.id = 'NODE_ID'
+        mock_load.return_value = node
+
+        action = node_action.NodeAction('NODE_ID', 'NODE_OPERATION', self.ctx,
+                                        cause='RPC Request')
+        action.id = 'ACTION_ID'
+        action.data = {
+            'status': policy_mod.CHECK_OK,
+            'reason': 'Policy checking passed'
+        }
+        action.inputs = {'operation': 'stop', 'params': {}}
+
+        mock_acquire.return_value = 'ACTION_ID'
+        mock_acquire_node.return_value = True
+
+        res_code, res_msg = action.execute()
+
+        reason = "Node operation 'stop' succeeded."
+        self.assertEqual(action.RES_OK, res_code)
+        self.assertEqual(reason, res_msg)
+        mock_load.assert_called_once_with(action.context, node_id='NODE_ID')
+        mock_acquire.assert_called_once_with(self.ctx, 'FAKE_CLUSTER',
+                                             'ACTION_ID', None,
+                                             lock.NODE_SCOPE, False)
+        policy_calls = [
+            mock.call('FAKE_CLUSTER', 'BEFORE'),
+            mock.call('FAKE_CLUSTER', 'AFTER')
+        ]
+        mock_release.assert_called_once_with('FAKE_CLUSTER', 'ACTION_ID',
+                                             lock.NODE_SCOPE)
+        mock_check.assert_has_calls(policy_calls)
+        mock_acquire_node.assert_called_once_with(self.ctx, 'NODE_ID',
+                                                  'ACTION_ID', None, True)
+        mock_release_node.assert_called_once_with('NODE_ID', 'ACTION_ID')
+
     @mock.patch.object(lock, 'cluster_lock_acquire')
     @mock.patch.object(lock, 'cluster_lock_release')
     @mock.patch.object(base_action.Action, 'policy_check')
@@ -848,7 +880,11 @@ class NodeActionTest(base.SenlinTestCase):
         mock_acquire_node.assert_called_once_with(self.ctx, 'NODE_ID',
                                                   'ACTION_ID', None, False)
         mock_release_node.assert_called_once_with('NODE_ID', 'ACTION_ID')
-        mock_check.assert_called_once_with('FAKE_CLUSTER', 'BEFORE')
+        check_calls = [
+            mock.call('FAKE_CLUSTER', 'BEFORE'),
+            mock.call('FAKE_CLUSTER', 'AFTER')
+        ]
+        mock_check.assert_has_calls(check_calls)
 
     @mock.patch.object(lock, 'cluster_lock_acquire')
     @mock.patch.object(lock, 'cluster_lock_release')
