@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
 
 from senlin.engine import registry
 from senlin.tests.unit.common import base
@@ -200,10 +200,22 @@ class RegistryTest(base.SenlinTestCase):
 
     def test_get_types(self):
         reg = registry.Registry('GLOBAL', None)
-        plugin1 = mock.Mock()
-        reg.register_plugin('FOO', plugin1)
-        plugin2 = mock.Mock()
-        reg.register_plugin('BAR', plugin2)
+        plugin1 = mock.Mock(VERSIONS={'1.0': 'bar'})
+        reg.register_plugin('FOO-1.0', plugin1)
+        plugin2 = mock.Mock(VERSIONS={'1.1': 'car'})
+        reg.register_plugin('BAR-1.1', plugin2)
 
-        self.assertIn({'name': 'FOO'}, reg.get_types())
-        self.assertIn({'name': 'BAR'}, reg.get_types())
+        self.assertIn(
+            {
+                'name': 'FOO',
+                'version': '1.0',
+                'support_status': {'1.0': 'bar'}
+            },
+            reg.get_types())
+        self.assertIn(
+            {
+                'name': 'BAR',
+                'version': '1.1',
+                'support_status': {'1.1': 'car'}
+            },
+            reg.get_types())
